<?php
if ( !defined( 'ABSPATH' ) ) exit;

if ( !class_exists( 'rtbNewGateway' ) ) {
/**
 * This class is responsible for payment processing via this gateway
 * 
 * @since 2.3.0
 */
class rtbNewGateway implements rtbPaymentGateway
{

  public static $gateway_identifier = 'new_gateway';

  private static $_instance;

  /**
   * 
   * Constructor of the class
   * It will be called only once
   * 
   * */
  private final function __construct()
  {
    $this->register_hooks();
  }

  /**
   * Get single instance of the class
   * 
   * @return rtbNewGateway instance
   */
  public static function get_instance()
  {
    if( ! isset( self::$_instance ) ) {
      self::$_instance = new rtbNewGateway();
    }
    
    return self::$_instance;
  }

  /**
   * Register the gateway by adding it to the available gateway list
   * */
  public static function register_gateway (array $gateway_list )
  {
    return array_merge(
      $gateway_list,
      [
        self::$gateway_identifier => [
          'label'    => __( 'New Gateway', 'restaurant-reservations-new-gateway' ),
          'instance' => self::get_instance()
        ]
      ]
    );
  }

  public function register_hooks()
  {
    add_action( 'rtb_booking_form_init', [$this, 'process_payment'] );

    // AJAX communication
    add_action( 'wp_ajax_new_gateway_action_1', array( $this, 'action_1' ) );
    add_action( 'wp_ajax_nopriv_new_gateway_action_1', array( $this, 'action_1' ) );

    
    add_filter( 'rtb_booking_metadata_defaults', [$this, 'default_booking_new_gateway_info'], 30, 1 );
    add_action( 'rtb_booking_load_post_data', [$this, 'populate_booking_new_gateway_info'], 30, 2 );
    add_filter( 'rtb_insert_booking_metadata', [$this, 'save_booking_new_gateway_info'], 30, 2 );

    /**
     * Adding info in the bookings table for admin
     */
    add_filter( 'rtb_admin_bookings_list_row_classes', [$this, 'add_class'], 30, 2 );
    add_filter( 'rtb_bookings_table_column_details', [$this, 'add_details'], 30, 2 );
    add_action( 'rtb_payment_summary', [$this, 'payment_summary'] );
  }

  /**
   * Talk to your AJAX request
   * */
  public function action_1() {}

  /**
   * Set $booking's default new-gateway meta information
   * 
   * @param  rtbBooking $booking
   */
  public function default_booking_new_gateway_info( $info_list )
  {
    $info_list['new_gateway_customer_id'] = 'ABC';

    return $info_list;
  }

  /**
   * Repopulate $booking with new-gateway meta information
   * 
   * @param  rtbBooking $booking
   */
  public function populate_booking_new_gateway_info( $booking )
  {
    $meta = get_post_meta( $booking->ID, 'rtb', true );

    if ( is_array( $meta ) && isset( $meta['new_gateway_customer_id'] ) ) {
      $booking->new_gateway_customer_id = $meta['new_gateway_customer_id'];
    }
  }

  /**
   * Store $booking default new-gateway meta information
   * 
   * @param  arrray $meta
   * @param  rtbBooking $booking
   */
  public function save_booking_new_gateway_info( $meta, $booking )
  {
    if ( isset( $booking->new_gateway_customer_id ) && !empty( $booking->new_gateway_customer_id ) ) {
      $meta['new_gateway_customer_id'] = $booking->new_gateway_customer_id;
    }

    return $meta;
  }

  /**
   * Add the CSS class to admin booking listing
   * 
   * @param array $row_class_list css classes for the row
   * @param rtbBooking $booking
   */
  public function add_class($row_class_list, $booking)
  {
    $paid_via_this_gateway = isset( $booking->new_gateway_customer_id );
    
    if( $paid_via_this_gateway ) {
      $row_class_list[] = 'paid-via-new-gateway';
    }
    return $row_class_list;
  }

  /**
   * Add gateway information in the admin bookin details pop-up
   * 
   * @param array $details Lable/value item array
   * @param rtbBooking $booking
   */
  public function add_details($details, $booking)
  {
    $paid_via_this_gateway = isset( $booking->new_gateway_customer_id );
    
    if( $paid_via_this_gateway ) {
      $details[] = array(
        'label' => __('New Gateway Customer ID', 'restaurant-reservations-new-gateway'),
        'value' => $booking->new_gateway_customer_id
      );
    }

    return $details;
  }

  /**
   * Add something after the payment summary info
   * */
  public function payment_summary()
  {
    global $rtb_controller;

    if ( self::$gateway_identifier == $rtb_controller->payment_manager->get_gateway_in_use() ) {
      echo '<p class="new-gateway">' . __( 'Message related to your payment gateway to customers.', 'restaurant-reservations-new-gateway' ) . '</p>';
    }
  }

  /**
   * Display your form/buttons etc to acquire the payment
   */
  public function print_payment_form( $booking )
  {
    global $rtb_controller;

    // Function alias
    $_gs = [$rtb_controller->settings, 'get_setting'];

    $nonce = wp_nonce_field( 'rtb-new-gateway-payment', 'nonce', true, false );
    $submit_btn_lbl = esc_html( $rtb_controller->settings->get_setting( 'label-make-deposit' ) );

    echo <<<HTML

      <div class="new-gateway-section">
        <p>You payment form or button here. You should load any JS, CSS and HTML resources here which are needed for the payment process.</p>
        <form 
          action='#' 
          method='POST' 
          id='stripe-payment-form' 
          data-booking_id='<?php echo esc_attr( $booking->ID ) ;?>'>
          
          {$nonce}
          <input type='hidden' name='action' value='rtb_new_gateway_payment'/>
          <input type='hidden' name='booking_id' value='{$booking->ID}' />
          
          <button type='submit'>
            {$submit_btn_lbl}
          </button>
        </form>
      </div>
    HTML;
  }

  /**
   * Maybe process payment
   * 
   * @return void Redirect and exit or display
   */
  public function process_payment()
  {
    global $rtb_controller;

    if ( ! isset( $_POST['action'] ) || 'rtb_new_gateway_payment' != $_POST['action'] ||  ! isset( $_POST['booking_id'] ) ) {
      return;
    }

    // Function alias
    $_gs = [$rtb_controller->settings, 'get_setting'];
    $booking_id = intval( $_POST['booking_id'] );

    // Define the form's action parameter
    $booking_page = $_gs( 'booking-page' );
    if ( ! empty( $booking_page ) ) {
      $booking_page = get_permalink( $booking_page );
    }
    else {
     $booking_page = get_permalink();
    }

    if ( !wp_verify_nonce( sanitize_text_field( $_POST['nonce'] ), 'rtb-new-gateway-payment' ) ) {
      // Invalid request
      $redirect = add_query_arg(
        array(
          'payment' => 'failed',
          'booking_id' => $booking_id,
          'error_code' => urlencode( __( 'The request has been rejected because it does not appear to have come from this site.', 'restaurant-reservations-new-gateway' ) )
        ),
        $booking_page
      );

      // redirect back to our previous page with the added query variable
      wp_redirect($redirect); exit;
    }

    /**
     * 
     * Process payment here and decide here to redirect or what to do save and what not to.
     * 
     * */


    // Load the booking
    require_once( RTB_PLUGIN_DIR . '/includes/Booking.class.php' );
    $booking = new rtbBooking();
    $booking->load_post( $booking_id );

    // Add/remove data from booking related to payment
    $booking->deposit = '10';
    $booking->receipt_id = 'SomeUniqueID';

    // and update booking status payment as being paid
    $booking->payment_paid();

    // redirect on successful payment
    $redirect = add_query_arg(
      array(
        'payment' => 'paid',
        'booking_id' => $booking_id
      ),
      $booking_page
    );

    // redirect
    wp_redirect($redirect); exit;
  }

}

}